@@ -3,7 +3,7 @@
 with 
   _ -> ());
 
-let dbm =
+let _ =
   Dbm.opendbm "itest-aux/testdb" [ Dbm.Dbm_rdwr; Dbm.Dbm_create ] 0o777 in
 
 print_string "OK\n";;
