@@ -1,4 +1,4 @@
-(* $Id: findlib.ml 225 2014-07-15 22:44:36Z gerd $
+(* $Id: findlib.ml 241 2014-10-14 09:55:08Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -136,7 +136,7 @@ let init
 
   let config_preds =
     match toolchain with
-      | None -> []
+      | None -> (try [Sys.getenv "OCAMLFIND_TOOLCHAIN"] with Not_found -> [])
       | Some p -> [p] in
 
   let sys_ocamlc, sys_ocamlopt, sys_ocamlcp, sys_ocamloptp, sys_ocamlmklib,
