@@ -1,4 +1,4 @@
-(* $Id: frontend.ml 231 2014-09-06 18:03:16Z gerd $
+(* $Id: frontend.ml 239 2014-10-14 09:39:42Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -551,7 +551,7 @@ let process_ppx_spec predicates packages ppx_opts =
     package_deep_ancestors predicates packages in
 
   let ppx_opts =
-    List.map 
+    List.map
       (fun opt ->
          match Fl_split.in_words opt with
            | pkg :: ((_ :: _) as opts) ->
@@ -559,25 +559,60 @@ let process_ppx_spec predicates packages ppx_opts =
                  try ignore(package_directory pkg); true
                  with No_such_package _ -> false in
                if not exists then
-                 failwith ("The package named in -ppxopt does not exist: " ^ 
+                 failwith ("The package named in -ppxopt does not exist: " ^
                              pkg);
                pkg, opts
-           | _ -> 
+           | _ ->
                failwith "-ppxopt must include package name, e.g. -ppxopt \"foo,-name bar\""
       )
       ppx_opts in
 
+  let meta_ppx_opts =
+    List.concat
+      (List.map
+        (fun pname ->
+          try
+            let opts = package_property predicates pname "ppxopt" in
+            (* Split by whitespace to get (package,options) combinations.
+               Then, split by commas to get individual options. *)
+            List.map
+              (fun opts ->
+                match Fl_split.in_words opts with
+                | pkg :: ((_ :: _) as opts) ->
+                    let exists =
+                      try ignore(package_directory pkg); true
+                      with No_such_package _ -> false in
+                    if not exists then
+                      failwith ("The package named in ppxopt variable does not exist: " ^
+                                  pkg ^ " (from " ^ pname ^ ")");
+                    let base = package_directory pname in
+                    pkg, List.map (resolve_path ~base ~explicit:true) opts
+                | _ ->
+                    failwith ("ppxopt variable must include package name, e.g. " ^
+                              "ppxopt=\"foo,-name bar\" (from " ^ pname ^ ")")
+              )
+              (Fl_split.in_words_ws opts)
+          with Not_found -> []
+        )
+        ppx_packages
+      ) in
+
   List.flatten
-    (List.map 
+    (List.map
        (fun pname ->
           let base = package_directory pname in
           let options =
-            try  List.assoc pname ppx_opts
-            with Not_found -> [] in
+            try
+              List.concat
+                (List.map (fun (_, opts) -> opts)
+                  (List.filter (fun (pname', _) -> pname' = pname)
+                    (meta_ppx_opts @ ppx_opts)))
+            with Not_found -> []
+          in
           try
             let preprocessor =
               resolve_path
-                ~base ~explicit:true 
+                ~base ~explicit:true
                 (package_property predicates pname "ppx") in
             ["-ppx"; String.concat " " (preprocessor :: options)]
           with Not_found -> []
